/*
 * GWT - General Windowing Toolkit
 *
 * Copyright (C) 1998 MenTaLguY - mentalg@geocities.com
 *                    Rodolphe Ortalo - ortalo@laas.fr
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Log: events.c,v $
 * Revision 1.8  2000/03/13 22:02:55  ortalo
 * Committed new version of LibGWT corresponding to 0.0.2.
 * The event management API is probably more or less defined. The simplest
 * approach finally won. (I should have remembered: KISS!)
 *
 * Revision 1.7  1999/05/23 23:56:43  marcus
 * Fixed broken includes.
 *
 * Revision 1.6  1999/05/20 20:34:22  ortalo
 * Several bugs corrections and small improvements.
 *
 * Revision 1.5  1999/05/13 20:01:38  ortalo
 * New version of libgwt. Now using LibGII for input management.
 * Some drawing functions implemented. Visibility management should
 * be rather okay. Still some work to do, but now, your eyes can
 * look at some drawings... not only code. ;-)
 *
 * Revision 1.4  1998/11/17 23:28:08  ortalo
 * Brought libgwt under autoconf&automake&libtool, in conformance with
 * the GNU configure and building system (and also, the rest of GGI. :-)
 * Should build and run as before (i.e.: not so much... :-(.)
 *
 * Revision 1.3  1998/08/23 18:57:08  ortalo
 * Minor editing (mainly comments).
 *
 * Revision 1.2  1998/08/11 22:17:33  ortalo
 * Adapted to the gwt.h->ggi/gwt.h change. Incorporated the modifications
 * brought by Tristan Wibberley.
 *
 * Revision 1.1  1998/07/09 18:38:39  ortalo
 * Initial integration of libgwt in the GGI repository.
 * This is experimental code, for your eyes only !
 * (Don't execute... ;-)
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include <errno.h>
#include <string.h>

#include "internal.h"
#include "gwt_debug.h"
#include <ggi/gii.h>
#include <ggi/gwt.h>

/************************************************************************
 *                      PRIVATE STATIC FUNCTIONS                        *
 ************************************************************************/
/*
 * Searches in all the direct child of a window the
 * ones that should receive the event (in the end).
 *  - mouse events are selected on location
 *  - keyboard events are selected on focuss
 *  - other events, we'll see... (TODO:)
 * Returns: a child window, or NULL if none found.
 */
/* TODO: Take into account the event mask of the window 
   TODO: to stop the propagation !! */
gwt_window_t gwt_child_window_search(gwt_window_t parent, gii_event *ev)
{
  gwt_window_t found = NULL;
  /* Demultiplex events among sub-windows */
  switch (ev->any.type)
    {
      /* Keys are tested according the focuss window */
    case evKeyPress:
    case evKeyRepeat:
      /* TODO: Should test that previous press is in same win? */
    case evKeyRelease: /* TODO: Similar check ? -- ortalo */
      found = parent->focuss; /* NULL if no focuss */
      break;
    case evPtrRelative:
    case evPtrAbsolute:
    case evPtrButtonPress:
    case evPtrButtonRelease:
      {
	gwt_window_t child = GWT_Z_TOP_CHILD(parent);
	while (child != NULL) {
	  if (gwt_region_contains_point(child->potentially_visible,
					parent->ptr_x,parent->ptr_y)) {
	    /* Recurse the search in that window: */
	    found = child;
	    /* and finish */
	    child = NULL;
	  } else {
	    child = GWT_Z_DOWN(child); /* iterate */
	    if (child == NULL) {
	      /* Well, none of the child is concerned */
	      found = NULL;
	    }
	  }
	}
      }
      break;
    default:
      /* Default result is NULL */
      break;
    }
  return found;
}

/************************************************************************
 *                        PUBLIC API FUNCTIONS                          *
 ************************************************************************/
/*
 * Obtain/Set the absolute pointer position
 */
int gwtGetPointerLocation(gwt_window_t win, int *x, int *y)
{
  *x = win->ptr_x;
  *y = win->ptr_y;
  return 0; /* Hard to fail */
}
int gwtSetPointerLocation(gwt_window_t win, int x, int y)
{
  win->ptr_x = x;
  win->ptr_y = y;
  return 0; /* Hard to fail */
}
/*
 * Poll the next event in a window
 */
int gwtEventPoll(gwt_window_t win, gii_event_mask mask, struct timeval *t)
{
  /* Return the state found */
  return giiEventPoll(win->input, mask, t);
}
/*
 * Obtain the next event from ONE (parent) window
 */
int gwtEventRead(gwt_window_t win, gii_event *ev, gii_event_mask mask)
{
  /* Return the event(s) found */
  int err = giiEventRead(win->input, ev, mask);
  /* Maintain absolute pointer position */
  switch (ev->any.type)
    {
    case evPtrAbsolute:
      win->ptr_x = ev->pmove.x;
      win->ptr_y = ev->pmove.y;
      break;
    case evPtrRelative:
      win->ptr_x += ev->pmove.x;
      win->ptr_y += ev->pmove.y;
      break;
    }
  return err;
}
/*
 * Send an event to ONE (parent) window
 */
int gwtEventSend(gwt_window_t win, gii_event *ev)
{
  /* Send the event */
  if (win->input != NULL)
    return giiEventSend(win->input, ev);
  else
    return -1;
}
/*
 * Obtain the event window associated to one
 * specific event of a parent window.
 *  - the window is selected based on the pointer
 *  location in case of pointer event;
 *  - the window is selected based on focuss
 *  situation in case of a keyboard event;
 *  - other events, we'll see... (TODO:)
 */
int gwtEventLocate(gwt_window_t parent, gii_event *ev, gwt_window_t *event_window)
{
  (*event_window) = gwt_child_window_search(parent, ev);
  return GGI_OK;
}
/*
 * Set the focussed sub-window
 */
int gwtSetFocus(gwt_window_t parent, gwt_window_t new_f)
{
  parent->focuss = new_f;
  return 0; /* Hard to fail...*/
}
/*
 * Get the focussed sub-window
 */
int gwtGetFocus(gwt_window_t parent, gwt_window_t *focussed)
{
  (*focussed) = parent->focuss;
  return 0;
}
